@@ -30,9 +30,10 @@ kable = Kable({
 })
 
 
-@app.route('/authenticate')
+@app.route('/authenticate/<string:path>')
 @kable.authenticate
-def authenticated():
+def authenticated(path):
+    print(path)
     return "hello world - authenticate"
 
 @app.route('/record')
